// Filename:	bbsstring.h
// Contents:	Those strings that couldn't be changed within the language
//		object.
// Author:	Greg Shaw
// Created:	12/8/94

#ifndef _BBSSTRING_H_
#define _BBSSTRING_H_

// the sysexec object is situated below the language object, which means 
// it can't take advantage of the language object.

#define EXPIRE3MINUTES  "\n\nYour time expires in 3 minutes.\n"
#define	PREPARETOLOGOFF	"Please prepare to log off.\n"
#define	YOUWILLBEOFF	"\n\nYou will be logged off in less than one minute.\n\n"
#define	PLEASELOGOFF	"Please log off.\n"

// bbsint lies below the language object 

#define CONTINUE 	"       \\o<continue>\\n"
#define QTOQUIT         "\\oPress '\\f7q\\n\\o' to quit, space bar to page\\n"

// rochatd:

#define CHATDHEADER "User         Alias         Messages        On since\n"
#define HASLEFTROOM  "%s has left the room."

#endif // _BBSSTRING_H_
