// Filename:   bbs.h
// Contents:   the bbs object definitions
// Author:     Greg Shaw
// Created:        5/28/93

/*

This file is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; either version 2, or (at your option) any
later version.

In addition to the permissions in the GNU General Public License, the
Free Software Foundation gives you unlimited permission to link the
compiled version of this file with other programs, and to distribute
those programs without any restriction coming from the use of this
file.  (The General Public License restrictions do apply in other
respects; for example, they cover modification of the file, and
distribution when not linked into another program.)

This file is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; see the file COPYING.  If not, write to
the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */

#ifndef _BBS_H_
#define _BBS_H_

// the low flags number for system message shown flag
#define SYSMSGFLAGS    0xE0000000
#define MSGFLAGS    0x20000000  // the low flags number for system message shown flag
#define MENU_STACK_SIZE    30   // 30 menus max stack depth
#define MENU_NAME_LEN  255      // 255 chars max per menu name
#define SDESC_LEN  61           // short description (for files)
#define MAX_COLOR_STRING   2048 // longest output string (after ANSIizing)
#undef  HAVE_RESTRICTED_BASH    // bash (re)compiled with -r option
// available?
#define  ONE_LANGUAGE_ONLY      // support only one language?

#endif                          // _BBS_H_







